@@ -77,7 +77,8 @@ static void             clipman_menu_get_property       (GObject *object,
                                                          GValue *value,
                                                          GParamSpec *pspec);
 #ifdef HAVE_QRENCODE
-GdkPixbuf *             clipman_menu_qrcode             (char *text);
+GdkPixbuf *             clipman_menu_qrcode             (char *text,
+                                                         gint scale_factor);
 #endif
 
 
@@ -128,12 +129,12 @@ cb_set_qrcode (GtkMenuItem *mi, const GdkPixbuf *pixbuf)
 #endif
 
 static void
-cb_set_clipboard_from_primary (GtkMenuItem *mi)
+cb_set_clipboard_from_primary (GtkMenuItem *mi, GObject *menu)
 {
   GtkClipboard *clipboard;
 
   clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-  gtk_clipboard_set_text (clipboard, gtk_menu_item_get_label (mi), -1);
+  gtk_clipboard_set_text (clipboard, g_object_get_data (menu, "selection-primary"), -1);
 }
 
 static void
@@ -293,12 +294,9 @@ cb_launch_clipman_bin (ClipmanMenu *menu,
 {
   GError *error = NULL;
   GtkWidget *error_dialog;
-  GtkWidget *mi = GTK_WIDGET (user_data);
+  gchar *command = user_data;
 
-  if (g_strcmp0 (gtk_menu_item_get_label (GTK_MENU_ITEM (mi)), "_Show full history...") == 0)
-    g_spawn_command_line_async ("xfce4-clipman-history", &error);
-  else
-    g_spawn_command_line_async ("xfce4-clipman-settings", &error);
+  g_spawn_command_line_async (command, &error);
 
   if (error != NULL)
   {
@@ -328,7 +326,6 @@ _clipman_menu_adjust_geometry (ClipmanMenu *menu)
 static void
 _clipman_menu_update_list (ClipmanMenu *menu)
 {
-  GtkClipboard *clipboard;
   GtkWidget *mi, *image;
 #ifdef HAVE_QRENCODE
   GdkPixbuf *pixbuf;
@@ -338,9 +335,16 @@ _clipman_menu_update_list (ClipmanMenu *menu)
   GSList *list, *l;
   gint pos = 0;
   guint i = 0;
-  gchar *selection_primary;
-  gchar *selection_clipboard;
+  const gchar *selection_primary;
+  const gchar *selection_clipboard;
+  gchar *selection_primary_short;
   gboolean skip_primary = FALSE;
+  cairo_surface_t *surface;
+  gint scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (menu));
+
+  /* retrieve clipboard and primary selections */
+  selection_clipboard = g_object_get_data (G_OBJECT (menu), "selection-clipboard");
+  selection_primary = g_object_get_data (G_OBJECT (menu), "selection-primary");
 
   /* Get the most recent item in the history */
   item_to_restore = clipman_history_get_item_to_restore (menu->priv->history);
@@ -356,12 +360,6 @@ _clipman_menu_update_list (ClipmanMenu *menu)
   if (menu->priv->reverse_order)
     list = g_slist_reverse (list);
 
-  clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-  selection_clipboard = gtk_clipboard_wait_for_text (clipboard);
-
-  clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
-  selection_primary = gtk_clipboard_wait_for_text (clipboard);
-
   for (i = 0, l = list; i < menu->priv->max_menu_items; i++, l = l->next)
     {
       if (l == NULL)
@@ -379,7 +377,9 @@ G_GNUC_END_IGNORE_DEPRECATIONS
         case CLIPMAN_HISTORY_TYPE_IMAGE:
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           mi = gtk_image_menu_item_new ();
-          image = gtk_image_new_from_pixbuf (item->preview.image);
+          surface = gdk_cairo_surface_create_from_pixbuf (item->preview.image, scale_factor, NULL);
+          image = gtk_image_new_from_surface (surface);
+          cairo_surface_destroy (surface);
           gtk_container_add (GTK_CONTAINER (mi), image);
 G_GNUC_END_IGNORE_DEPRECATIONS
           break;
@@ -438,12 +438,14 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       gtk_menu_shell_insert (GTK_MENU_SHELL (menu), mi, pos++);
       gtk_widget_show_all (mi);
 
-      if ((pixbuf = clipman_menu_qrcode (item_to_restore->content.text)) != NULL)
+      if ((pixbuf = clipman_menu_qrcode (item_to_restore->content.text, scale_factor)) != NULL)
         {
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           mi = gtk_image_menu_item_new ();
 G_GNUC_END_IGNORE_DEPRECATIONS
-          gtk_container_add (GTK_CONTAINER (mi), gtk_image_new_from_pixbuf (pixbuf));
+          surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale_factor, NULL);
+          gtk_container_add (GTK_CONTAINER (mi), gtk_image_new_from_surface (surface));
+          cairo_surface_destroy (surface);
           g_signal_connect (mi, "activate", G_CALLBACK (cb_set_qrcode), pixbuf);
           menu->priv->list = g_slist_prepend (menu->priv->list, mi);
           gtk_menu_shell_insert (GTK_MENU_SHELL (menu), mi, pos++);
@@ -486,19 +488,25 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
       if (skip_primary == FALSE)
         {
+          selection_primary_short = clipman_common_shorten_preview (selection_primary);
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-          mi = gtk_image_menu_item_new_with_label (selection_primary);
+          mi = gtk_image_menu_item_new_with_label (selection_primary_short);
           image = gtk_image_new_from_icon_name ("input-mouse-symbolic", GTK_ICON_SIZE_MENU);
           gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
 G_GNUC_END_IGNORE_DEPRECATIONS
+          g_free (selection_primary_short);
           gtk_menu_shell_insert (GTK_MENU_SHELL (menu), mi, 0);
           gtk_widget_show_all (mi);
-          g_signal_connect (mi, "activate", G_CALLBACK (cb_set_clipboard_from_primary), NULL);
+          g_signal_connect (mi, "activate", G_CALLBACK (cb_set_clipboard_from_primary), menu);
           menu->priv->list = g_slist_prepend (menu->priv->list, mi);
+
+          if (pos == 0)
+            {
+              /* Set the clear history item sensitive */
+              gtk_widget_set_sensitive (menu->priv->mi_clear_history, TRUE);
+            }
         }
-      g_free (selection_primary);
     }
-  g_free (selection_clipboard);
 
   _clipman_menu_adjust_geometry(menu);
 }
@@ -588,6 +596,10 @@ clipman_menu_init (ClipmanMenu *menu)
 
   /* ClipmanHistory */
   menu->priv->history = clipman_history_get ();
+  clipman_history_set_scale_factor (menu->priv->history, NULL, GTK_WIDGET (menu));
+  g_signal_connect_object (menu, "notify::scale-factor",
+                           G_CALLBACK (clipman_history_set_scale_factor),
+                           menu->priv->history, G_CONNECT_SWAPPED);
 
   /* Connect signal on show to update the items */
   g_signal_connect_swapped (menu, "show", G_CALLBACK (_clipman_menu_update_list), menu);
@@ -601,7 +613,7 @@ clipman_menu_init (ClipmanMenu *menu)
     {
       mi = gtk_menu_item_new_with_mnemonic (_("_Show full history..."));
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-      g_signal_connect_swapped (mi, "activate", G_CALLBACK (cb_launch_clipman_bin), mi);
+      g_signal_connect (mi, "activate", G_CALLBACK (cb_launch_clipman_bin), "xfce4-clipman-history");
     }
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -614,7 +626,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
   mi = gtk_menu_item_new_with_mnemonic (_("_Clipman settings..."));
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect_swapped (mi, "activate", G_CALLBACK (cb_launch_clipman_bin), mi);
+  g_signal_connect (mi, "activate", G_CALLBACK (cb_launch_clipman_bin), "xfce4-clipman-settings");
 
   /* Show all the items */
   gtk_widget_show_all (GTK_WIDGET (menu));
@@ -697,11 +709,12 @@ clipman_menu_get_property (GObject *object,
 
 #ifdef HAVE_QRENCODE
 GdkPixbuf *
-clipman_menu_qrcode (char *text)
+clipman_menu_qrcode (char *text,
+                     gint scale_factor)
 {
 	QRcode *qrcode;
 	GdkPixbuf *pixbuf, *pixbuf_scaled;
-	int i, j, k, rowstride, channels;
+	int i, j, k, rowstride, channels, size;
 	guchar *pixel;
 	unsigned char *data;
 
@@ -726,7 +739,8 @@ clipman_menu_qrcode (char *text)
 			data++;
 		}
 
-	pixbuf_scaled = gdk_pixbuf_scale_simple (pixbuf, (qrcode->width + 2) * 3, (qrcode->width + 2) * 3, GDK_INTERP_NEAREST);
+	size = (qrcode->width + 2) * 3 * scale_factor;
+	pixbuf_scaled = gdk_pixbuf_scale_simple (pixbuf, size, size, GDK_INTERP_NEAREST);
 
 	QRcode_free(qrcode);
 	g_object_unref(pixbuf);
