@@ -60,7 +60,7 @@ clipboard_manager_ownership_exists (void)
  */
 
 MyPlugin *
-plugin_register (void)
+plugin_register (gboolean panel_plugin)
 {
   MyPlugin *plugin = g_slice_new0 (MyPlugin);
   GError *error = NULL;
@@ -76,23 +76,29 @@ plugin_register (void)
     }
 
   plugin->app = gtk_application_new ("org.xfce.clipman", 0);
-  g_application_register (G_APPLICATION (plugin->app), NULL, &error);
-  if (error != NULL)
+
+  /* all this has already been done in main-status-icon.c in the case of a status icon */
+  if (panel_plugin)
     {
-      g_warning ("Unable to register GApplication: %s", error->message);
-      g_error_free (error);
-      error = NULL;
+      g_application_register (G_APPLICATION (plugin->app), NULL, &error);
+      if (error != NULL)
+        {
+          g_warning ("Unable to register GApplication: %s", error->message);
+          g_error_free (error);
+          error = NULL;
+        }
+
+      if (g_application_get_is_remote (G_APPLICATION (plugin->app)))
+        {
+          g_message ("Primary instance org.xfce.clipman already running");
+          clipman_common_show_info_dialog ();
+          g_object_unref (plugin->app);
+          return NULL;
+        }
+
+      g_set_application_name (_("Clipman"));
     }
 
-  if (g_application_get_is_remote (G_APPLICATION (plugin->app)))
-    {
-      g_message ("Primary instance org.xfce.clipman already running");
-      clipman_common_show_info_dialog ();
-      g_object_unref (plugin->app);
-      return FALSE;
-    }
-
-  g_set_application_name (_("Clipman"));
   g_signal_connect_swapped (plugin->app, "activate", G_CALLBACK (plugin_popup_menu), plugin);
 
   /* Xfconf */
@@ -119,6 +125,8 @@ plugin_register (void)
   plugin->collector = clipman_collector_get ();
   xfconf_g_property_bind (plugin->channel, "/settings/add-primary-clipboard",
                           G_TYPE_BOOLEAN, plugin->collector, "add-primary-clipboard");
+  xfconf_g_property_bind (plugin->channel, "/settings/persistent-primary-clipboard",
+                          G_TYPE_BOOLEAN, plugin->collector, "persistent-primary-clipboard");
   xfconf_g_property_bind (plugin->channel, "/settings/history-ignore-primary-clipboard",
                           G_TYPE_BOOLEAN, plugin->collector, "history-ignore-primary-clipboard");
   xfconf_g_property_bind (plugin->channel, "/settings/enable-actions",
@@ -172,13 +180,13 @@ plugin_load (MyPlugin *plugin)
   while (TRUE)
     {
       filename = g_strdup_printf ("%s/xfce4/clipman/image%d.png", g_get_user_cache_dir (), i++);
+      DBG ("Loading image from cache file %s", filename);
       image = gdk_pixbuf_new_from_file (filename, NULL);
       g_unlink (filename);
       g_free (filename);
       if (image == NULL)
         break;
 
-      DBG ("Loading image from cache file %s", filename);
       clipman_history_add_image (plugin->history, image);
       g_object_unref (image);
     }
@@ -241,7 +249,7 @@ plugin_save (MyPlugin *plugin)
   list = g_slist_reverse (list);
   if (list != NULL)
     {
-      texts = g_malloc0 (g_slist_length (list) * sizeof (gchar *));
+      texts = g_new0 (const gchar *, g_slist_length (list));
       for (n_texts = n_images = 0, l = list; l != NULL; l = l->next)
         {
           item = l->data;
@@ -334,7 +342,7 @@ plugin_about (MyPlugin *plugin)
                          "logo-icon-name", "xfce4-clipman-plugin",
                          "comments", _("Clipboard Manager for Xfce"),
                          "version", PACKAGE_VERSION,
-                         "copyright", "Copyright © 2003-2016 The Xfce development team",
+                         "copyright", "Copyright © 2003-2023 The Xfce development team",
                          "license", license,
                          "website", "https://docs.xfce.org/panel-plugins/xfce4-clipman-plugin",
                          "website-label", "docs.xfce.org",
@@ -362,9 +370,17 @@ plugin_configure (MyPlugin *plugin)
   }
 }
 
-void
-plugin_popup_menu (MyPlugin *plugin)
+static void
+plugin_popup_menu_real (GtkClipboard *clipboard,
+                        const gchar  *text,
+                        gpointer      data)
 {
+  MyPlugin *plugin = data;
+
+  /* store primary selection for later use */
+  g_object_set_data_full (G_OBJECT (plugin->menu), "selection-primary",
+                          g_strdup (text), g_free);
+
   if (xfconf_channel_get_bool (plugin->channel, "/tweaks/popup-at-pointer", FALSE))
     {
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -395,3 +411,28 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 #endif
   }
 }
+
+static void
+plugin_popup_menu_primary (GtkClipboard *clipboard,
+                           const gchar  *text,
+                           gpointer      data)
+{
+  MyPlugin *plugin = data;
+
+  /* store clipboard selection for later use */
+  g_object_set_data_full (G_OBJECT (plugin->menu), "selection-clipboard",
+                          g_strdup (text), g_free);
+
+  /* request primary selection */
+  gtk_clipboard_request_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+                              plugin_popup_menu_real, plugin);
+}
+
+void
+plugin_popup_menu (MyPlugin *plugin)
+{
+  /* first, request text of each clipboard asynchronously, to not dispatch
+   * current event associated to menu popup */
+  gtk_clipboard_request_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
+                              plugin_popup_menu_primary, plugin);
+}
