@@ -83,6 +83,8 @@ prop_dialog_run (void)
   /* Behavior tab: General */
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "add-selections")),
                                 DEFAULT_ADD_PRIMARY_CLIPBOARD);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "persistent-selections")),
+                                DEFAULT_PERSISTENT_PRIMARY_CLIPBOARD);
 #ifdef HAVE_QRENCODE
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "show-qr-code")),
                                 DEFAULT_SHOW_QR_CODE);
@@ -105,6 +107,8 @@ prop_dialog_run (void)
 #endif
   xfconf_g_property_bind (xfconf_channel, "/settings/add-primary-clipboard", G_TYPE_BOOLEAN,
                           gtk_builder_get_object (builder, "add-selections"), "active");
+  xfconf_g_property_bind (xfconf_channel, "/settings/persistent-primary-clipboard", G_TYPE_BOOLEAN,
+                          gtk_builder_get_object (builder, "persistent-selections"), "active");
   xfconf_g_property_bind (xfconf_channel, "/tweaks/paste-on-activate",
                           G_TYPE_UINT, G_OBJECT (combobox), "active");
 
@@ -229,11 +233,10 @@ cb_show_help (GtkButton *button)
 {
   gchar *docpath = NULL;
   gchar *command = NULL;
-
-  /* Find localized documentation path on disk */
-#ifdef ENABLE_NLS
   gchar *locale = NULL;
   gchar *offset;
+
+  /* Find localized documentation path on disk */
 #ifdef HAVE_LOCALE_H
   locale = g_strdup (setlocale (LC_MESSAGES, ""));
   if (locale != NULL)
@@ -271,9 +274,6 @@ cb_show_help (GtkButton *button)
     }
 
   g_free (locale);
-#else
-  docpath = g_strdup (DOCDIR"/html/C/index.html");
-#endif
 
   /* Revert to online documentation if not available on disk */
   if (g_file_test (docpath, G_FILE_TEST_EXISTS))
